class Player {
    constructor(game, spawnPoint) {
        if (!spawnPoint) {
            spawnPoint = new BABYLON.Vector3(0,10,-10);
        }

        this.spawnPoint = spawnPoint;
        this.scene = game.scene;
        this.game = game;
        this.height = 2;
        this.speed = 1;
        this.inertia = 0.9;
        this.angularInertia = 0;
        // The mouse sensibility (lower the better sensible)
        this.angularSensibility = 1000;
        this.camera = this._initCamera();
        // The player must click on the canvas to activate control
        this.controlEnabled = false;
        this.weapon = new Weapon(game, this);

        var canvas = this.scene.getEngine().getRenderingCanvas();
        // Event listener on click on the canvas
        canvas.addEventListener("click", (evt) => {
            var width = this.scene.getEngine().getRenderWidth();
            var height = this.scene.getEngine().getRenderHeight();

            if (this.controlEnabled) {
                var pickInfo = this.scene.pick(width/2, height/2, null, false, this.camera);
                this.handleUserMouse(evt, pickInfo);
            }
        }, false);

        // Event listener to go pointer lock
        this._initPointerLock();

        // The representation of player in the minimap
        var s = BABYLON.Mesh.CreateSphere("player2", 16, 4, this.scene);
        s.position.y = 10;
        s.registerBeforeRender(() => {
            s.position.x = this.camera.position.x;
            s.position.z = this.camera.position.z;
        });

        var red = new BABYLON.StandardMaterial("red", this.scene);
        red.diffuseColor = BABYLON.Color3.Red();
        red.specularColor = BABYLON.Color3.Black();
        s.material = red;
        s.layerMask = 1;

        // Set the active camera for the minimap
        this.scene.activeCameras.push(this.camera);
        this.scene.activeCamera = this.camera;
    };

    _initPointerLock() {
        // Request pointer lock
        var canvas = this.scene.getEngine().getRenderingCanvas();
        canvas.addEventListener("click", (evt) => {
            canvas.requestPointerLock = canvas.requestPointerLock || canvas.msRequestPointerLock || canvas.mozRequestPointerLock || canvas.webkitRequestPointerLock;
            if (canvas.requestPointerLock) {
                canvas.requestPointerLock();
            }
        }, false);

        // Event listener when the pointerlock is updated.
        var pointerlockchange = (event) => {
            this.controlEnabled = (document.mozPointerLockElement === canvas || document.webkitPointerLockElement === canvas || document.msPointerLockElement === canvas || document.pointerLockElement === canvas);
            if (!this.controlEnabled) {
                this.camera.detachControl(canvas);
            } else {
                this.camera.attachControl(canvas);
            }
        };
        document.addEventListener("pointerlockchange", pointerlockchange, false);
        document.addEventListener("mspointerlockchange", pointerlockchange, false);
        document.addEventListener("mozpointerlockchange", pointerlockchange, false);
        document.addEventListener("webkitpointerlockchange", pointerlockchange, false);
    }

    /**
     * Init the player camera
     * @returns {BABYLON.FreeCamera}
     * @private
     */
    _initCamera() {

        var cam = new BABYLON.FreeCamera("camera", this.spawnPoint, this.scene);
        cam.attachControl(this.scene.getEngine().getRenderingCanvas());
        cam.ellipsoid = new BABYLON.Vector3(2, this.height, 2);
        cam.checkCollisions = true;
        cam.applyGravity = true;
        // WASD
        cam.keysUp = [87]; // W
        cam.keysDown = [83]; // S
        cam.keysLeft = [65]; // A
        cam.keysRight = [68]; // D
        cam.speed = this.speed;
        cam.inertia = this.inertia;
        cam.angularInertia = this.angularInertia;
        cam.angularSensibility = this.angularSensibility;
        cam.layerMask = 2;

        return cam;
    }

    /**
     * Handle the user input on keyboard
     * @param keycode
     */
    handleUserKeyboard(keycode) {
        switch (keycode) {

        }
    }

    /**
     * Handle the user input on mouse.
     * click = shoot
     * @param evt
     * @param pickInfo The pick data retrieved when the click has been done
     */
    handleUserMouse(evt, pickInfo) {
        this.weapon.fire(pickInfo);
    }
};